@@ -1,9 +1,10 @@
 /*
- * Copyright  2000-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -26,6 +27,7 @@ import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.ExitException;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
+import org.apache.tools.ant.ExitStatusException;
 import org.apache.tools.ant.types.Commandline;
 import org.apache.tools.ant.types.CommandlineJava;
 import org.apache.tools.ant.types.Environment;
@@ -35,6 +37,8 @@ import org.apache.tools.ant.types.Reference;
 import org.apache.tools.ant.types.Assertions;
 import org.apache.tools.ant.types.Permissions;
 import org.apache.tools.ant.types.RedirectorElement;
+import org.apache.tools.ant.taskdefs.condition.Os;
+import org.apache.tools.ant.util.KeepAliveInputStream;
 
 /**
  * Launcher for Java applications. Allows use of
@@ -60,19 +64,39 @@ public class Java extends Task {
     private File input;
     private File output;
     private File error;
-    
+
+    // CheckStyle:VisibilityModifier OFF - bc
     protected Redirector redirector = new Redirector(this);
     protected RedirectorElement redirectorElement;
-    
+    // CheckStyle:VisibilityModifier ON
+
     private String resultProperty;
     private Permissions perm = null;
 
     private boolean spawn = false;
     private boolean incompatibleWithSpawn = false;
+
+    private static final String TIMEOUT_MESSAGE = 
+        "Timeout: killed the sub-process";
+
+    /**
+     * Normal constructor
+     */
+    public Java() {
+    }
+
+    /**
+     * create a bound task
+     * @param owner owner
+     */
+    public Java(Task owner) {
+        bindToOwner(owner);
+    }
+
     /**
      * Do the execution.
      * @throws BuildException if failOnError is set to true and the application
-     * returns a non 0 result code
+     * returns a nonzero result code.
      */
     public void execute() throws BuildException {
         File savedDir = dir;
@@ -80,10 +104,13 @@ public class Java extends Task {
 
         int err = -1;
         try {
+            checkConfiguration();
             err = executeJava();
             if (err != 0) {
                 if (failOnError) {
-                    throw new BuildException("Java returned: " + err, getLocation());
+                    throw new ExitStatusException("Java returned: " + err,
+                            err,
+                            getLocation());
                 } else {
                     log("Java Result: " + err, Project.MSG_ERR);
                 }
@@ -99,17 +126,25 @@ public class Java extends Task {
      * Do the execution and return a return code.
      *
      * @return the return code from the execute java class if it was
-     * executed in a separate VM (fork = "yes").
+     * executed in a separate VM (fork = "yes") or a security manager was
+     * installed that prohibits ExitVM (default).
      *
-     * @throws BuildException if required parameters are missing
+     * @throws BuildException if required parameters are missing.
      */
     public int executeJava() throws BuildException {
-        String classname = cmdl.getClassname();
-        if (classname == null && cmdl.getJar() == null) {
+        return executeJava(getCommandLine());
+    }
+
+    /**
+     * Check configuration.
+     * @throws BuildException if required parameters are missing.
+     */
+    protected void checkConfiguration() throws BuildException {
+        String classname = getCommandLine().getClassname();
+        if (classname == null && getCommandLine().getJar() == null) {
             throw new BuildException("Classname must not be null.");
         }
-
-        if (!fork && cmdl.getJar() != null) {
+        if (!fork && getCommandLine().getJar() != null) {
             throw new BuildException("Cannot execute a jar in non-forked mode."
                                      + " Please set fork='true'. ");
         }
@@ -117,26 +152,30 @@ public class Java extends Task {
             throw new BuildException("Cannot spawn a java process in non-forked mode."
                                      + " Please set fork='true'. ");
         }
+        if (getCommandLine().getClasspath() != null
+            && getCommandLine().getJar() != null) {
+            log("When using 'jar' attribute classpath-settings are ignored. "
+                + "See the manual for more information.", Project.MSG_VERBOSE);
+        }
         if (spawn && incompatibleWithSpawn) {
             getProject().log("spawn does not allow attributes related to input, "
             + "output, error, result", Project.MSG_ERR);
             getProject().log("spawn also does not allow timeout", Project.MSG_ERR);
-            getProject().log( "finally, spawn is not compatible "
+            getProject().log("finally, spawn is not compatible "
                 + "with a nested I/O <redirector>", Project.MSG_ERR);
             throw new BuildException("You have used an attribute "
                 + "or nested element which is not compatible with spawn");
         }
-        if (cmdl.getAssertions() != null && !fork) {
+        if (getCommandLine().getAssertions() != null && !fork) {
             log("Assertion statements are currently ignored in non-forked mode");
         }
-
         if (fork) {
             if (perm != null) {
                 log("Permissions can not be set this way in forked mode.", Project.MSG_WARN);
             }
-            log(cmdl.describeCommand(), Project.MSG_VERBOSE);
+            log(getCommandLine().describeCommand(), Project.MSG_VERBOSE);
         } else {
-            if (cmdl.getVmCommand().size() > 1) {
+            if (getCommandLine().getVmCommand().size() > 1) {
                 log("JVM args ignored when same JVM is used.",
                     Project.MSG_WARN);
             }
@@ -144,71 +183,85 @@ public class Java extends Task {
                 log("Working directory ignored when same JVM is used.",
                     Project.MSG_WARN);
             }
-
             if (newEnvironment || null != env.getVariables()) {
                 log("Changes to environment variables are ignored when same "
                     + "JVM is used.", Project.MSG_WARN);
             }
-
-            if (cmdl.getBootclasspath() != null) {
+            if (getCommandLine().getBootclasspath() != null) {
                 log("bootclasspath ignored when same JVM is used.",
                     Project.MSG_WARN);
             }
-            if (perm == null && failOnError == true) {
+            if (perm == null) {
                 perm = new Permissions(true);
-                log("running " + this.cmdl.getClassname()
+                log("running " + this.getCommandLine().getClassname()
                     + " with default permissions (exit forbidden)", Project.MSG_VERBOSE);
             }
-            log("Running in same VM " + cmdl.describeJavaCommand(),
+            log("Running in same VM " + getCommandLine().describeJavaCommand(),
                 Project.MSG_VERBOSE);
         }
-
         setupRedirector();
+    }
+
+    /**
+     * Execute the specified CommandlineJava.
+     * @param commandLine CommandLineJava instance.
+     * @return the exit value of the process if forked, 0 otherwise.
+     */
+    protected int executeJava(CommandlineJava commandLine) {
         try {
             if (fork) {
                 if (!spawn) {
-                    return fork(cmdl.getCommandline());
+                    return fork(commandLine.getCommandline());
                 } else {
-                    spawn(cmdl.getCommandline());
+                    spawn(commandLine.getCommandline());
                     return 0;
                 }
             } else {
                 try {
-                    run(cmdl);
+                    run(commandLine);
                     return 0;
                 } catch (ExitException ex) {
                     return ex.getStatus();
                 }
             }
         } catch (BuildException e) {
+            if (e.getLocation() == null && getLocation() != null) {
+                e.setLocation(getLocation());
+            }
             if (failOnError) {
                 throw e;
             } else {
-                log(e);
-                return 0;
+                if (TIMEOUT_MESSAGE.equals(e.getMessage())) {
+                    log(TIMEOUT_MESSAGE);
+                } else {
+                    log(e);
+                }
+                return -1;
             }
+        } catch (ThreadDeath t) {
+            throw t; // cf. NB #47191
         } catch (Throwable t) {
             if (failOnError) {
-                throw new BuildException(t);
+                throw new BuildException(t, getLocation());
             } else {
                 log(t);
-                return 0;
+                return -1;
             }
         }
     }
 
     /**
-     * set whether or not you want the process to be spawned
-     * default is not spawned
-     * @param spawn if true you do not want ant to wait for the end of the process
-     * @since ant 1.6
+     * Set whether or not you want the process to be spawned;
+     * default is not spawned.
+     * @param spawn if true you do not want Ant to wait for the end of the process.
+     * @since Ant 1.6
      */
     public void setSpawn(boolean spawn) {
         this.spawn = spawn;
     }
 
     /**
-     * Set the classpath to be used when running the Java class
+     * Set the classpath to be used when running the Java class.
      *
      * @param s an Ant Path object containing the classpath.
      */
@@ -217,103 +270,114 @@ public class Java extends Task {
     }
 
     /**
-     * Adds a path to the classpath.
+     * Add a path to the classpath.
      *
-     * @return created classpath
+     * @return created classpath.
      */
     public Path createClasspath() {
-        return cmdl.createClasspath(getProject()).createPath();
+        return getCommandLine().createClasspath(getProject()).createPath();
     }
 
     /**
-     * Adds a path to the bootclasspath.
+     * Add a path to the bootclasspath.
      * @since Ant 1.6
      *
-     * @return created bootclasspath
+     * @return created bootclasspath.
      */
     public Path createBootclasspath() {
-        return cmdl.createBootclasspath(getProject()).createPath();
+        return getCommandLine().createBootclasspath(getProject()).createPath();
     }
 
     /**
-     * Sets the permissions for the application run inside the same JVM.
+     * Set the permissions for the application run inside the same JVM.
      * @since Ant 1.6
-     * @return .
+     * @return Permissions.
      */
     public Permissions createPermissions() {
-        if (perm == null) {
-            perm = new Permissions();
-        }
+        perm = (perm == null) ? new Permissions() : perm;
         return perm;
     }
 
     /**
-     * Classpath to use, by reference.
+     * Set the classpath to use by reference.
      *
-     * @param r a reference to an existing classpath
+     * @param r a reference to an existing classpath.
      */
     public void setClasspathRef(Reference r) {
         createClasspath().setRefid(r);
     }
 
     /**
-     * The location of the JAR file to execute.
+     * Set the location of the JAR file to execute.
      *
-     * @param jarfile the jarfile that one wants to execute
+     * @param jarfile the jarfile to execute.
      *
-     * @throws BuildException if there is also a main class specified
+     * @throws BuildException if there is also a main class specified.
      */
     public void setJar(File jarfile) throws BuildException {
-        if (cmdl.getClassname() != null) {
+        if (getCommandLine().getClassname() != null) {
             throw new BuildException("Cannot use 'jar' and 'classname' "
                                      + "attributes in same command.");
         }
-        cmdl.setJar(jarfile.getAbsolutePath());
+        getCommandLine().setJar(jarfile.getAbsolutePath());
     }
 
     /**
-     * Sets the Java class to execute.
+     * Set the Java class to execute.
      *
-     * @param s the name of the main class
+     * @param s the name of the main class.
      *
-     * @throws BuildException if the jar attribute has been set
+     * @throws BuildException if the jar attribute has been set.
      */
     public void setClassname(String s) throws BuildException {
-        if (cmdl.getJar() != null) {
+        if (getCommandLine().getJar() != null) {
             throw new BuildException("Cannot use 'jar' and 'classname' "
                                      + "attributes in same command");
         }
-        cmdl.setClassname(s);
+        getCommandLine().setClassname(s);
     }
 
     /**
      * Deprecated: use nested arg instead.
      * Set the command line arguments for the class.
      *
-     * @param s arguments
+     * @param s arguments.
      *
      * @ant.attribute ignore="true"
      */
     public void setArgs(String s) {
         log("The args attribute is deprecated. "
             + "Please use nested arg elements.", Project.MSG_WARN);
-        cmdl.createArgument().setLine(s);
+        getCommandLine().createArgument().setLine(s);
     }
 
     /**
-     * Adds a command-line argument.
+     * If set, system properties will be copied to the cloned VM--as
+     * well as the bootclasspath unless you have explicitly specified
+     * a bootclaspath.
      *
-     * @return created argument
+     * <p>Doesn't have any effect unless fork is true.</p>
+     * @param cloneVm if true copy system properties.
+     * @since Ant 1.7
+     */
+    public void setCloneVm(boolean cloneVm) {
+        getCommandLine().setCloneVm(cloneVm);
+    }
+
+    /**
+     * Add a command-line argument.
+     *
+     * @return created argument.
      */
     public Commandline.Argument createArg() {
-        return cmdl.createArgument();
+        return getCommandLine().createArgument();
     }
 
     /**
-     * The name of a property in which the return code of the
+     * Set the name of the property in which the return code of the
      * command should be stored. Only of interest if failonerror=false.
      *
-     * @param resultProperty name of property
+     * @param resultProperty name of property.
      *
      * @since Ant 1.6
      */
@@ -323,8 +387,8 @@ public class Java extends Task {
     }
 
     /**
-     * helper method to set result property to the
-     * passed in value if appropriate
+     * Helper method to set result property to the
+     * passed in value if appropriate.
      *
      * @param result the exit code
      */
@@ -347,58 +411,58 @@ public class Java extends Task {
     /**
      * Set the command line arguments for the JVM.
      *
-     * @param s jvmargs
+     * @param s jvmargs.
      */
     public void setJvmargs(String s) {
         log("The jvmargs attribute is deprecated. "
             + "Please use nested jvmarg elements.", Project.MSG_WARN);
-        cmdl.createVmArgument().setLine(s);
+        getCommandLine().createVmArgument().setLine(s);
     }
 
     /**
      * Adds a JVM argument.
      *
-     * @return JVM argument created
+     * @return JVM argument created.
      */
     public Commandline.Argument createJvmarg() {
-        return cmdl.createVmArgument();
+        return getCommandLine().createVmArgument();
     }
 
     /**
      * Set the command used to start the VM (only if forking).
      *
-     * @param s command to start the VM
+     * @param s command to start the VM.
      */
     public void setJvm(String s) {
-        cmdl.setVm(s);
+        getCommandLine().setVm(s);
     }
 
     /**
-     * Adds a system property.
+     * Add a system property.
      *
-     * @param sysp system property
+     * @param sysp system property.
      */
     public void addSysproperty(Environment.Variable sysp) {
-        cmdl.addSysproperty(sysp);
+        getCommandLine().addSysproperty(sysp);
     }
 
     /**
-     * Adds a set of properties as system properties.
+     * Add a set of properties as system properties.
      *
-     * @param sysp set of properties to add
+     * @param sysp set of properties to add.
      *
      * @since Ant 1.6
      */
     public void addSyspropertyset(PropertySet sysp) {
-        cmdl.addSyspropertyset(sysp);
+        getCommandLine().addSyspropertyset(sysp);
     }
 
     /**
      * If true, then fail if the command exits with a
-     * returncode other than 0
+     * returncode other than zero.
      *
-     * @param fail if true fail the build when the command exits with a non
-     * zero returncode
+     * @param fail if true fail the build when the command exits with a
+     * nonzero returncode.
      */
     public void setFailonerror(boolean fail) {
         failOnError = fail;
@@ -406,9 +470,9 @@ public class Java extends Task {
     }
 
     /**
-     * The working directory of the process
+     * Set the working directory of the process.
      *
-     * @param d working directory
+     * @param d working directory.
      *
      */
     public void setDir(File d) {
@@ -416,9 +480,9 @@ public class Java extends Task {
     }
 
     /**
-     * File the output of the process is redirected to.
+     * Set the File to which the output of the process is redirected.
      *
-     * @param out name of the output file
+     * @param out the output File.
      */
     public void setOutput(File out) {
         this.output = out;
@@ -426,9 +490,9 @@ public class Java extends Task {
     }
 
     /**
-     * Set the input to use for the task
+     * Set the input to use for the task.
      *
-     * @param input name of the input file
+     * @param input name of the input file.
      */
     public void setInput(File input) {
         if (inputString != null) {
@@ -440,9 +504,9 @@ public class Java extends Task {
     }
 
     /**
-     * Set the string to use as input
+     * Set the string to use as input.
      *
-     * @param inputString the string which is used as the input source
+     * @param inputString the string which is used as the input source.
      */
     public void setInputString(String inputString) {
         if (input != null) {
@@ -454,12 +518,12 @@ public class Java extends Task {
     }
 
     /**
-     * Controls whether error output of exec is logged. This is only useful
+     * Set whether error output of exec is logged. This is only useful
      * when output is being redirected and error output is desired in the
-     * Ant log
+     * Ant log.
      *
      * @param logError get in the ant log the messages coming from stderr
-     * in the case that fork = true
+     * in the case that fork = true.
      */
     public void setLogError(boolean logError) {
         redirector.setLogError(logError);
@@ -467,11 +531,11 @@ public class Java extends Task {
     }
 
     /**
-     * File the error stream of the process is redirected to.
+     * Set the File to which the error stream of the process is redirected.
      *
-     * @param error file getting the error stream
+     * @param error file getting the error stream.
      *
-     * @since ant 1.6
+     * @since Ant 1.6
      */
     public void setError(File error) {
         this.error = error;
@@ -479,10 +543,10 @@ public class Java extends Task {
     }
 
     /**
-     * Property name whose value should be set to the output of
+     * Set the property name whose value should be set to the output of
      * the process.
      *
-     * @param outputProp property name
+     * @param outputProp property name.
      *
      */
     public void setOutputproperty(String outputProp) {
@@ -491,12 +555,12 @@ public class Java extends Task {
     }
 
     /**
-     * Property name whose value should be set to the error of
+     * Set the property name whose value should be set to the error of
      * the process.
      *
-     * @param errorProperty property name
+     * @param errorProperty property name.
      *
-     * @since ant 1.6
+     * @since Ant 1.6
      */
     public void setErrorProperty(String errorProperty) {
         redirector.setErrorProperty(errorProperty);
@@ -506,26 +570,26 @@ public class Java extends Task {
     /**
      * Corresponds to -mx or -Xmx depending on VM version.
      *
-     * @param max max memory parameter
+     * @param max max memory parameter.
      */
     public void setMaxmemory(String max) {
-        cmdl.setMaxmemory(max);
+        getCommandLine().setMaxmemory(max);
     }
 
     /**
-     * Sets the JVM version.
-     * @param value JVM version
+     * Set the JVM version.
+     * @param value JVM version.
      */
     public void setJVMVersion(String value) {
-        cmdl.setVmversion(value);
+        getCommandLine().setVmversion(value);
     }
 
     /**
-     * Adds an environment variable.
+     * Add an environment variable.
      *
      * <p>Will be ignored if we are not forking a new VM.
      *
-     * @param var new environment variable
+     * @param var new environment variable.
      *
      * @since Ant 1.5
      */
@@ -549,7 +613,7 @@ public class Java extends Task {
     /**
      * If true, append output to existing file.
      *
-     * @param append if true, append output to existing file
+     * @param append if true, append output to existing file.
      *
      * @since Ant 1.5
      */
@@ -559,9 +623,9 @@ public class Java extends Task {
     }
 
     /**
-     * Timeout in milliseconds after which the process will be killed.
+     * Set the timeout in milliseconds after which the process will be killed.
      *
-     * @param value time out in milliseconds
+     * @param value timeout in milliseconds.
      *
      * @since Ant 1.5
      */
@@ -571,34 +635,33 @@ public class Java extends Task {
     }
 
     /**
-     *  assertions to enable in this program (if fork=true)
+     * Add assertions to enable in this program (if fork=true).
+     * @param asserts assertion set.
      * @since Ant 1.6
-     * @param asserts assertion set
      */
     public void addAssertions(Assertions asserts) {
-        if(cmdl.getAssertions() != null) {
+        if (getCommandLine().getAssertions() != null) {
             throw new BuildException("Only one assertion declaration is allowed");
         }
-        cmdl.setAssertions(asserts);
+        getCommandLine().setAssertions(asserts);
     }
 
     /**
-     * Add a <CODE>RedirectorElement</CODE> to this task.
-     * @param redirectorElement   <CODE>RedirectorElement</CODE>.
+     * Add a <code>RedirectorElement</code> to this task.
+     * @param redirectorElement   <code>RedirectorElement</code>.
      */
     public void addConfiguredRedirector(RedirectorElement redirectorElement) {
         if (this.redirectorElement != null) {
-            throw new BuildException("cannot have > 1 nested <redirector>s");
-        } else {
-            this.redirectorElement = redirectorElement;
-            incompatibleWithSpawn = true;
+            throw new BuildException("cannot have > 1 nested redirectors");
         }
+        this.redirectorElement = redirectorElement;
+        incompatibleWithSpawn = true;
     }
 
     /**
      * Pass output sent to System.out to specified output file.
      *
-     * @param output a string of output on its way to the handlers
+     * @param output a string of output on its way to the handlers.
      *
      * @since Ant 1.5
      */
@@ -611,30 +674,27 @@ public class Java extends Task {
     }
 
     /**
-     * Handle an input request by this task
+     * Handle an input request by this task.
      *
      * @param buffer the buffer into which data is to be read.
      * @param offset the offset into the buffer at which data is stored.
-     * @param length the amount of data to read
+     * @param length the amount of data to read.
      *
-     * @return the number of bytes read
+     * @return the number of bytes read.
      *
-     * @exception IOException if the data cannot be read
+     * @exception IOException if the data cannot be read.
      * @since Ant 1.6
      */
     public int handleInput(byte[] buffer, int offset, int length)
         throws IOException {
-        if (redirector.getInputStream() != null) {
-            return redirector.handleInput(buffer, offset, length);
-        } else {
-            return super.handleInput(buffer, offset, length);
-        }
+        // Should work whether or not redirector.inputStream == null:
+        return redirector.handleInput(buffer, offset, length);
     }
 
     /**
      * Pass output sent to System.out to specified output file.
      *
-     * @param output string of output on its way to its handlers
+     * @param output string of output on its way to its handlers.
      *
      * @since Ant 1.5.2
      */
@@ -647,9 +707,9 @@ public class Java extends Task {
     }
 
     /**
-     * Handle output sent to System.err
+     * Handle output sent to System.err.
      *
-     * @param output string of stderr
+     * @param output string of stderr.
      *
      * @since Ant 1.5
      */
@@ -664,7 +724,7 @@ public class Java extends Task {
     /**
      * Handle output sent to System.err and flush the stream.
      *
-     * @param output string of stderr
+     * @param output string of stderr.
      *
      * @since Ant 1.5.2
      */
@@ -672,7 +732,7 @@ public class Java extends Task {
         if (redirector.getErrorStream() != null) {
             redirector.handleErrorFlush(output);
         } else {
-            super.handleErrorOutput(output);
+            super.handleErrorFlush(output);
         }
     }
 
@@ -687,11 +747,17 @@ public class Java extends Task {
         if (redirectorElement != null) {
             redirectorElement.configure(redirector);
         }
+        if (!spawn && input == null && inputString == null) {
+            // #24918: send standard input to the process by default.
+            redirector.setInputStream(
+                new KeepAliveInputStream(getProject().getDefaultInputStream()));
+        }
     }
 
     /**
      * Executes the given classname with the given arguments as it
-     * was a command line application.
+     * were a command line application.
+     * @param command CommandlineJava.
      */
     private void run(CommandlineJava command) throws BuildException {
         try {
@@ -705,7 +771,7 @@ public class Java extends Task {
             exe.execute(getProject());
             redirector.complete();
             if (exe.killedProcess()) {
-                throw new BuildException("Timeout: killed the sub-process");
+                throw new BuildException(TIMEOUT_MESSAGE);
             }
         } catch (IOException e) {
             throw new BuildException(e);
@@ -714,39 +780,18 @@ public class Java extends Task {
 
     /**
      * Executes the given classname with the given arguments in a separate VM.
+     * @param command String[] of command-line arguments.
      */
     private int fork(String[] command) throws BuildException {
-
         Execute exe
             = new Execute(redirector.createHandler(), createWatchdog());
-        exe.setAntRun(getProject());
+        setupExecutable(exe, command);
 
-        if (dir == null) {
-            dir = getProject().getBaseDir();
-        } else if (!dir.exists() || !dir.isDirectory()) {
-            throw new BuildException(dir.getAbsolutePath()
-                                     + " is not a valid directory",
-                                     getLocation());
-        }
-
-        exe.setWorkingDirectory(dir);
-
-        String[] environment = env.getVariables();
-        if (environment != null) {
-            for (int i = 0; i < environment.length; i++) {
-                log("Setting environment variable: " + environment[i],
-                    Project.MSG_VERBOSE);
-            }
-        }
-        exe.setNewenvironment(newEnvironment);
-        exe.setEnvironment(environment);
-
-        exe.setCommandline(command);
         try {
             int rc = exe.execute();
             redirector.complete();
             if (exe.killedProcess()) {
-                throw new BuildException("Timeout: killed the sub-process");
+                throw new BuildException(TIMEOUT_MESSAGE);
             }
             return rc;
         } catch (IOException e) {
@@ -756,23 +801,37 @@ public class Java extends Task {
 
     /**
      * Executes the given classname with the given arguments in a separate VM.
+     * @param command String[] of command-line arguments.
      */
     private void spawn(String[] command) throws BuildException {
-
-        Execute exe
-            = new Execute();
-        exe.setAntRun(getProject());
-
-        if (dir == null) {
-            dir = getProject().getBaseDir();
-        } else if (!dir.exists() || !dir.isDirectory()) {
-            throw new BuildException(dir.getAbsolutePath()
-                                     + " is not a valid directory",
-                                     getLocation());
+        Execute exe = new Execute();
+        setupExecutable(exe, command);
+        try {
+            exe.spawn();
+        } catch (IOException e) {
+            throw new BuildException(e, getLocation());
         }
+    }
 
-        exe.setWorkingDirectory(dir);
+    /**
+     * Do all configuration for an executable that
+     * is common across the {@link #fork(String[])} and
+     * {@link #spawn(String[])} methods.
+     * @param exe executable.
+     * @param command command to execute.
+     */
+    private void setupExecutable(Execute exe, String[] command) {
+        exe.setAntRun(getProject());
+        setupWorkingDir(exe);
+        setupEnvironment(exe);
+        setupCommandLine(exe, command);
+    }
 
+    /**
+     * Set up our environment variables.
+     * @param exe executable.
+     */
+    private void setupEnvironment(Execute exe) {
         String[] environment = env.getVariables();
         if (environment != null) {
             for (int i = 0; i < environment.length; i++) {
@@ -782,21 +841,60 @@ public class Java extends Task {
         }
         exe.setNewenvironment(newEnvironment);
         exe.setEnvironment(environment);
+    }
 
-        exe.setCommandline(command);
-        try {
-            exe.spawn();
-        } catch (IOException e) {
-            throw new BuildException(e, getLocation());
+    /**
+     * Set the working dir of the new process.
+     * @param exe executable.
+     * @throws BuildException if the dir doesn't exist.
+     */
+    private void setupWorkingDir(Execute exe) {
+        if (dir == null) {
+            dir = getProject().getBaseDir();
+        } else if (!dir.exists() || !dir.isDirectory()) {
+            throw new BuildException(dir.getAbsolutePath()
+                                     + " is not a valid directory",
+                                     getLocation());
+        }
+        exe.setWorkingDirectory(dir);
+    }
+
+    /**
+     * Set the command line for the exe.
+     * On VMS, hands off to {@link #setupCommandLineForVMS(Execute, String[])}.
+     * @param exe executable.
+     * @param command command to execute.
+     */
+    private void setupCommandLine(Execute exe, String[] command) {
+        //On VMS platform, we need to create a special java options file
+        //containing the arguments and classpath for the java command.
+        //The special file is supported by the "-V" switch on the VMS JVM.
+        if (Os.isFamily("openvms")) {
+            setupCommandLineForVMS(exe, command);
+        } else {
+            exe.setCommandline(command);
         }
     }
+
     /**
-     * Executes the given classname with the given arguments as it
-     * was a command line application.
+     * On VMS platform, we need to create a special java options file
+     * containing the arguments and classpath for the java command.
+     * The special file is supported by the "-V" switch on the VMS JVM.
      *
-     * @param classname the name of the class to run
-     * @param args  arguments for the class
-     * @throws BuildException in case of IO Exception in the execution
+     * @param exe executable.
+     * @param command command to execute.
+     */
+    private void setupCommandLineForVMS(Execute exe, String[] command) {
+        ExecuteJava.setupCommandLineForVMS(exe, command);
+    }
+
+    /**
+     * Executes the given classname with the given arguments as if it
+     * were a command line application.
+     *
+     * @param classname the name of the class to run.
+     * @param args  arguments for the class.
+     * @throws BuildException in case of IOException in the execution.
      */
     protected void run(String classname, Vector args) throws BuildException {
         CommandlineJava cmdj = new CommandlineJava();
@@ -811,15 +909,15 @@ public class Java extends Task {
      * Clear out the arguments to this java task.
      */
     public void clearArgs() {
-        cmdl.clearJavaArgs();
+        getCommandLine().clearJavaArgs();
     }
 
     /**
      * Create the Watchdog to kill a runaway process.
      *
-     * @return new watchdog
+     * @return new watchdog.
      *
-     * @throws BuildException under unknown circumstances
+     * @throws BuildException under unknown circumstances.
      *
      * @since Ant 1.5
      */
@@ -831,6 +929,8 @@ public class Java extends Task {
     }
 
     /**
+     * Log the specified Throwable.
+     * @param t the Throwable to log.
      * @since 1.6.2
      */
     private void log(Throwable t) {
@@ -840,4 +940,24 @@ public class Java extends Task {
         w.close();
         log(sw.toString(), Project.MSG_ERR);
     }
+
+    /**
+     * Accessor to the command line.
+     *
+     * @return the current command line.
+     * @since 1.6.3
+     */
+    public CommandlineJava getCommandLine() {
+        return cmdl;
+    }
+
+    /**
+     * Get the system properties of the command line.
+     *
+     * @return the current properties of this java invocation.
+     * @since 1.6.3
+     */
+    public CommandlineJava.SysProperties getSysProperties() {
+        return getCommandLine().getSystemProperties();
+    }
 }
