@@ -31,6 +31,7 @@ import org.apache.tools.ant.DirectoryScanner;
 import org.apache.tools.ant.MagicNames;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.taskdefs.compilers.CompilerAdapter;
+import org.apache.tools.ant.taskdefs.compilers.CompilerAdapterExtension;
 import org.apache.tools.ant.taskdefs.compilers.CompilerAdapterFactory;
 import org.apache.tools.ant.types.Path;
 import org.apache.tools.ant.types.Reference;
@@ -79,6 +80,7 @@ public class Javac extends MatchingTask {
     private static final String FAIL_MSG
         = "Compile failed; see the compiler error output for details.";
 
+    private static final String JAVAC17 = "javac1.7";
     private static final String JAVAC16 = "javac1.6";
     private static final String JAVAC15 = "javac1.5";
     private static final String JAVAC14 = "javac1.4";
@@ -143,6 +145,8 @@ public class Javac extends MatchingTask {
             return JAVAC15;
         } else if (JavaEnvUtils.isJavaVersion(JavaEnvUtils.JAVA_1_6)) {
             return JAVAC16;
+        } else if (JavaEnvUtils.isJavaVersion(JavaEnvUtils.JAVA_1_7)) {
+            return JAVAC17;
         } else {
             return CLASSIC;
         }
@@ -594,7 +598,7 @@ public class Javac extends MatchingTask {
     /**
      * Sets the target VM that the classes will be compiled for. Valid
      * values depend on the compiler, for jdk 1.4 the valid values are
-     * "1.1", "1.2", "1.3", "1.4", "1.5", "1.6", "5" and "6".
+     * "1.1", "1.2", "1.3", "1.4", "1.5", "1.6", "1.7", "5", "6" and "7".
      * @param target the target VM
      */
     public void setTarget(String target) {
@@ -758,7 +762,8 @@ public class Javac extends MatchingTask {
     }
 
     private String getAltCompilerName(String anImplementation) {
-        if (JAVAC16.equalsIgnoreCase(anImplementation)
+        if (JAVAC17.equalsIgnoreCase(anImplementation)
+                || JAVAC16.equalsIgnoreCase(anImplementation)
                 || JAVAC15.equalsIgnoreCase(anImplementation)
                 || JAVAC14.equalsIgnoreCase(anImplementation)
                 || JAVAC13.equalsIgnoreCase(anImplementation)) {
@@ -770,7 +775,8 @@ public class Javac extends MatchingTask {
         }
         if (MODERN.equalsIgnoreCase(anImplementation)) {
             String nextSelected = assumedJavaVersion();
-            if (JAVAC16.equalsIgnoreCase(nextSelected)
+            if (JAVAC17.equalsIgnoreCase(nextSelected)
+                    || JAVAC16.equalsIgnoreCase(nextSelected)
                     || JAVAC15.equalsIgnoreCase(nextSelected)
                     || JAVAC14.equalsIgnoreCase(nextSelected)
                     || JAVAC13.equalsIgnoreCase(nextSelected)) {
@@ -807,7 +813,7 @@ public class Javac extends MatchingTask {
     }
 
     /**
-     * The property to set on compliation success.
+     * The property to set on compilation success.
      * This property will not be set if the compilation
      * fails, or if there are no files to compile.
      * @param updatedProperty the property name to use.
@@ -818,7 +824,7 @@ public class Javac extends MatchingTask {
     }
 
     /**
-     * The property to set on compliation failure.
+     * The property to set on compilation failure.
      * This property will be set if the compilation
      * fails.
      * @param errorProperty the property name to use.
@@ -929,23 +935,53 @@ public class Javac extends MatchingTask {
      */
     protected void scanDir(File srcDir, File destDir, String[] files) {
         GlobPatternMapper m = new GlobPatternMapper();
-        m.setFrom("*.java");
-        m.setTo("*.class");
-        SourceFileScanner sfs = new SourceFileScanner(this);
-        File[] newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
+        String[] extensions = findSupportedFileExtensions();
+        
+        for (int i = 0; i < extensions.length; i++) {
+            m.setFrom(extensions[i]);
+            m.setTo("*.class");
+            SourceFileScanner sfs = new SourceFileScanner(this);
+            File[] newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
 
-        if (newFiles.length > 0) {
-            lookForPackageInfos(srcDir, newFiles);
-            File[] newCompileList
-                = new File[compileList.length + newFiles.length];
-            System.arraycopy(compileList, 0, newCompileList, 0,
-                    compileList.length);
-            System.arraycopy(newFiles, 0, newCompileList,
-                    compileList.length, newFiles.length);
-            compileList = newCompileList;
+            if (newFiles.length > 0) {
+                lookForPackageInfos(srcDir, newFiles);
+                File[] newCompileList
+                    = new File[compileList.length + newFiles.length];
+                System.arraycopy(compileList, 0, newCompileList, 0,
+                                 compileList.length);
+                System.arraycopy(newFiles, 0, newCompileList,
+                                 compileList.length, newFiles.length);
+                compileList = newCompileList;
+            }
         }
     }
 
+    private String[] findSupportedFileExtensions() {
+        String compilerImpl = getCompiler();
+        CompilerAdapter adapter =
+            nestedAdapter != null ? nestedAdapter :
+            CompilerAdapterFactory.getCompiler(compilerImpl, this,
+                                               createCompilerClasspath());
+        String[] extensions = null;
+        if (adapter instanceof CompilerAdapterExtension) {
+            extensions =
+                ((CompilerAdapterExtension) adapter).getSupportedFileExtensions();
+        } 
+
+        if (extensions == null) {
+            extensions = new String[] { "java" };
+        }
+
+        // now process the extensions to ensure that they are the
+        // right format
+        for (int i = 0; i < extensions.length; i++) {
+            if (!extensions[i].startsWith("*.")) {
+                extensions[i] = "*." + extensions[i];
+            }
+        }
+        return extensions; 
+    }
+
     /**
      * Gets the list of files to be compiled.
      * @return the list of files as an array
@@ -965,6 +1001,7 @@ public class Javac extends MatchingTask {
     protected boolean isJdkCompiler(String compilerImpl) {
         return MODERN.equals(compilerImpl)
             || CLASSIC.equals(compilerImpl)
+            || JAVAC17.equals(compilerImpl)
             || JAVAC16.equals(compilerImpl)
             || JAVAC15.equals(compilerImpl)
             || JAVAC14.equals(compilerImpl)
